/*
  **
  ** conf.h - function prototypes for the config handling routines
  **
  ** Part of the drpoxy package by Matthew Pratt.
  **
  ** Copyright 2000 Jeroen Vreeken (pe1rxq@amsat.nl)
  **
  ** This software is licensed under the terms of the GNU General
  ** Public License (GPL). Please see the file COPYING for details.
  **
  **
*/
#ifndef CONF_H 
#define CONF_H 1

#include <config.h>

#include "tmdns.h"

#define CONF_PATH_LEN 256

#define MAX_ALSO_LOCAL 32

/* 
    more parameters may be added later.
 */
struct config {
  int  port;
  int  daemon_mode;
  int  no_fork;
  char service_file[CONF_PATH_LEN];
  char config_file[CONF_PATH_LEN];
  char debug_file[CONF_PATH_LEN];
  char * interfaces[MAX_IF + 1];
  /* ttl stuff */
  int  default_ttl;
  char username[CONF_PATH_LEN];
  char hostname[CONF_PATH_LEN];
  char pid_file[CONF_PATH_LEN] ;
  int  dns_bridge;
  int  dns_port;
  int  gather_delay;
  int  allow_nonlocal;
  char * also_local[MAX_ALSO_LOCAL + 1];
  char dynamic_service_file[CONF_PATH_LEN];
};

/** 
 * typedef for a param copy function. 
 */
typedef void (* conf_copy_func)(char *, void *) ;
typedef void (* conf_print_func)(FILE * fp, void *);

/**
 * description for parameters in the config file
 */
typedef struct {
  char * param_name;         /* name for this parameter             */
  char * comment;            /* a comment for this parameter        */
  void * conf_value;         /* pointer to a field in struct config */
  void * def_value;
  conf_copy_func  init;      /* a function to set the value in 'config'*/
  conf_copy_func  copy;      /* a function to set the value in 'config'*/
  conf_print_func print;     /* a function to print the value from 'config'*/
} config_param; 


extern struct config config;                      
int  conf_load (char *conf_file);
void conf_defaults (void);
void conf_cmdparse(char *cmd, char *arg1);
int  conf_bool (char *val);

void conf_print();

#endif /* CONF_H */
